/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : entry.s
Purpose : Entry point and vector table for the GD32VF103.

Additional information:
  [1] The RISC-V Instruction Set Manual Volume II: Privileged Architecture
      ...
      3.1.7 Machine Trap-Vector Base-Address Register (mtvec)
        ...
        When vectored interrupts are enabled, interrupt cause 0, which corresponds to user-mode software interrupts,
        are vectored to the same location as synchronous exceptions. This ambiguity does not arise in practice,
        since user-mode software interrupts are either disabled or delegated to user mode.
        ...
*/

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/

#if (__riscv_xlen == 64)
  #define STORE     sd
  #define LOAD      ld
  #define REGBYTES   8
#else
  #define STORE     sw
  #define LOAD      lw
  #define REGBYTES   4
#endif

/*********************************************************************
*
*       MACROS
*
**********************************************************************
*/

/*********************************************************************
*
*      STARTFUNC
*/
    .macro STARTFUNC FunctionName:req, SectionName=
        .ifb \SectionName                        // If parameter section name is blank
        .section .text.\FunctionName, "ax", %progbits
        .else
        .section \SectionName, "ax", %progbits
        .endif
        .type \FunctionName, "function"
        .global \FunctionName
        .balign 4
#if (!defined(__clang__))
        .func
#endif
\FunctionName:
    .endm

/*********************************************************************
*
*      ENDFUNC
*/
    .macro ENDFUNC FunctionName
#if (defined(__clang__))
\FunctionName\()_end:
        .size \FunctionName, \FunctionName\()_end-\FunctionName
#else
        .endfunc
#endif
    .endm

/*********************************************************************
*
*       Weak declarations
*
**********************************************************************
*/
        .weak ISR_U_Software
        .weak ISR_S_Software
        .weak ISR_U_Timer
        .weak ISR_S_Timer
        .weak ISR_U_External
        .weak ISR_S_External
        .weak ISR_M_External
        .weak eclic_msip_handler
        .weak eclic_mtip_handler
        .weak eclic_bwei_handler
        .weak eclic_pmovi_handler
        .weak WWDGT_IRQHandler
        .weak LVD_IRQHandler
        .weak TAMPER_IRQHandler
        .weak RTC_IRQHandler
        .weak FMC_IRQHandler
        .weak RCU_IRQHandler
        .weak EXTI0_IRQHandler
        .weak EXTI1_IRQHandler
        .weak EXTI2_IRQHandler
        .weak EXTI3_IRQHandler
        .weak EXTI4_IRQHandler
        .weak DMA0_Channel0_IRQHandler
        .weak DMA0_Channel1_IRQHandler
        .weak DMA0_Channel2_IRQHandler
        .weak DMA0_Channel3_IRQHandler
        .weak DMA0_Channel4_IRQHandler
        .weak DMA0_Channel5_IRQHandler
        .weak DMA0_Channel6_IRQHandler
        .weak ADC0_1_IRQHandler
        .weak CAN0_TX_IRQHandler
        .weak CAN0_RX0_IRQHandler
        .weak CAN0_RX1_IRQHandler
        .weak CAN0_EWMC_IRQHandler
        .weak EXTI5_9_IRQHandler
        .weak TIMER0_BRK_IRQHandler
        .weak TIMER0_UP_IRQHandler
        .weak TIMER0_TRG_CMT_IRQHandler
        .weak TIMER0_Channel_IRQHandler
        .weak TIMER1_IRQHandler
        .weak TIMER2_IRQHandler
        .weak TIMER3_IRQHandler
        .weak I2C0_EV_IRQHandler
        .weak I2C0_ER_IRQHandler
        .weak I2C1_EV_IRQHandler
        .weak I2C1_ER_IRQHandler
        .weak SPI0_IRQHandler
        .weak SPI1_IRQHandler
        .weak USART0_IRQHandler
        .weak USART1_IRQHandler
        .weak USART2_IRQHandler
        .weak EXTI10_15_IRQHandler
        .weak RTC_Alarm_IRQHandler
        .weak USBFS_WKUP_IRQHandler
        .weak EXMC_IRQHandler
        .weak TIMER4_IRQHandler
        .weak SPI2_IRQHandler
        .weak UART3_IRQHandler
        .weak UART4_IRQHandler
        .weak TIMER5_IRQHandler
        .weak TIMER6_IRQHandler
        .weak DMA1_Channel0_IRQHandler
        .weak DMA1_Channel1_IRQHandler
        .weak DMA1_Channel2_IRQHandler
        .weak DMA1_Channel3_IRQHandler
        .weak DMA1_Channel4_IRQHandler
        .weak CAN1_TX_IRQHandler
        .weak CAN1_RX0_IRQHandler
        .weak CAN1_RX1_IRQHandler
        .weak CAN1_EWMC_IRQHandler
        .weak USBFS_IRQHandler

/*********************************************************************
*
*       Vector table
*
**********************************************************************
*/
        .global vectors
        .section .vectors, "a", %progbits
        .balign 512  // mtvt, at least 64 byte aligned, for 65-128 entries 512 byte aligned
vectors:
        .word OS_TrapHandler  // This vector is ambiguously used for user mode software interrupts as well (see [1])
        .word ISR_S_Software
        .word 0
        .word ISR_M_Software
        .word ISR_U_Timer
        .word ISR_S_Timer
        .word 0
        .word ISR_M_Timer
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word eclic_bwei_handler
        .word eclic_pmovi_handler
        .word WWDGT_IRQHandler
        .word LVD_IRQHandler
        .word TAMPER_IRQHandler
        .word RTC_IRQHandler
        .word FMC_IRQHandler
        .word RCU_IRQHandler
        .word EXTI0_IRQHandler
        .word EXTI1_IRQHandler
        .word EXTI2_IRQHandler
        .word EXTI3_IRQHandler
        .word EXTI4_IRQHandler
        .word DMA0_Channel0_IRQHandler
        .word DMA0_Channel1_IRQHandler
        .word DMA0_Channel2_IRQHandler
        .word DMA0_Channel3_IRQHandler
        .word DMA0_Channel4_IRQHandler
        .word DMA0_Channel5_IRQHandler
        .word DMA0_Channel6_IRQHandler
        .word ADC0_1_IRQHandler
        .word CAN0_TX_IRQHandler
        .word CAN0_RX0_IRQHandler
        .word CAN0_RX1_IRQHandler
        .word CAN0_EWMC_IRQHandler
        .word EXTI5_9_IRQHandler
        .word TIMER0_BRK_IRQHandler
        .word TIMER0_UP_IRQHandler
        .word TIMER0_TRG_CMT_IRQHandler
        .word TIMER0_Channel_IRQHandler
        .word TIMER1_IRQHandler
        .word TIMER2_IRQHandler
        .word TIMER3_IRQHandler
        .word I2C0_EV_IRQHandler
        .word I2C0_ER_IRQHandler
        .word I2C1_EV_IRQHandler
        .word I2C1_ER_IRQHandler
        .word SPI0_IRQHandler
        .word SPI1_IRQHandler
        .word USART0_IRQHandler
        .word USART1_IRQHandler
        .word USART2_IRQHandler
        .word EXTI10_15_IRQHandler
        .word RTC_Alarm_IRQHandler
        .word USBFS_WKUP_IRQHandler
        .word 0
        .word 0
        .word 0
        .word 0
        .word 0
        .word EXMC_IRQHandler
        .word 0
        .word TIMER4_IRQHandler
        .word SPI2_IRQHandler
        .word UART3_IRQHandler
        .word UART4_IRQHandler
        .word TIMER5_IRQHandler
        .word TIMER6_IRQHandler
        .word DMA1_Channel0_IRQHandler
        .word DMA1_Channel1_IRQHandler
        .word DMA1_Channel2_IRQHandler
        .word DMA1_Channel3_IRQHandler
        .word DMA1_Channel4_IRQHandler
        .word 0
        .word 0
        .word CAN1_TX_IRQHandler
        .word CAN1_RX0_IRQHandler
        .word CAN1_RX1_IRQHandler
        .word CAN1_EWMC_IRQHandler
        .word USBFS_IRQHandler

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/
/*********************************************************************
*
*       Reset_Handler
*/
        STARTFUNC Reset_Handler, .app_entry
//
// Initialize the gp and tp registers
//
       .option  push
       .option  norelax
        lui     gp,     %hi(__global_pointer$)
        addi    gp, gp, %lo(__global_pointer$)
        lui     tp,     %hi(__thread_pointer$)
        addi    tp, tp, %lo(__thread_pointer$)
       .option  pop
//
// Use mtvec instead of mnvec for NMIs by setting mmisc_ctl[9] to 1
//
        li      t0, 0x200
        csrs    0x7d0, t0
//
// Initialize mtvt (vectored traps)
//
        la      t0, vectors
        csrw    mtvt, t0
//
// Initialize mtvec (non-vectored traps)
//
        la      t0, trap_entry
        csrw    mtvec, t0
//
// Jump to program start
//
        la      a0, _start
        jr      a0
        ENDFUNC Reset_Handler

/********************************************************************/

        .end

/*************************** End of file ****************************/
