/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : BSP for Freedom E31 Coreplex IP Evaluation Arty FPGA Dev Kit
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define NUMLEDS          (3)
#define LED_RED_MASK     (1u << 22)
#define LED_GREEN_MASK   (1u << 19)
#define LED_BLUE_MASK    (1u << 21)

#define GPIO_BASE_ADDR   (0x10012000u)
#define GPIO_OUTPUT_EN   (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x8u))
#define GPIO_OUTPUT_VAL  (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0xCu))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  GPIO_OUTPUT_VAL |= (LED_RED_MASK | LED_GREEN_MASK | LED_BLUE_MASK);  // Initially clear LED
  GPIO_OUTPUT_EN  |= (LED_RED_MASK | LED_GREEN_MASK | LED_BLUE_MASK);  // Enable GPIO output
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < NUMLEDS) {
    if (Index == 0) {
      GPIO_OUTPUT_VAL &= ~LED_BLUE_MASK;
    } else if (Index == 1) {
      GPIO_OUTPUT_VAL &= ~LED_GREEN_MASK;
    } else if (Index == 2) {
      GPIO_OUTPUT_VAL &= ~LED_RED_MASK;
    }
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < NUMLEDS) {
    if (Index == 0) {
      GPIO_OUTPUT_VAL |= LED_BLUE_MASK;
    } else if (Index == 1) {
      GPIO_OUTPUT_VAL |= LED_GREEN_MASK;
    } else if (Index == 2) {
      GPIO_OUTPUT_VAL |= LED_RED_MASK;
    }
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index < NUMLEDS) {
    if (Index == 0) {
      GPIO_OUTPUT_VAL ^= LED_BLUE_MASK;
    } else if (Index == 1) {
      GPIO_OUTPUT_VAL ^= LED_GREEN_MASK;
    } else if (Index == 2) {
      GPIO_OUTPUT_VAL ^= LED_RED_MASK;
    }
  }
}

/****** End Of File *************************************************/
