/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : BSP.c
Purpose : BSP for the SiFive HiFive IoT board
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define LED_ROW0  ( 9)  // LED_ROW0 GPIO23
#define LED_COL0  (22)  // LED_COL0 GPIO22
#define LED_COL1  (21)  // LED_COL1 GPIO21
#define LED_COL2  (19)  // LED_COL1 GPIO19
#define LED_COL3  ( 2)  // LED_COL1 GPIO2
#define LED_COL4  ( 1)  // LED_COL1 GPIO1

#define GPIO_BASE_ADDR   (0x10012000u)
#define GPIO_OUTPUT_EN   (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x8u))
#define GPIO_OUTPUT_VAL  (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0xCu))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  GPIO_OUTPUT_EN  |= (1u << LED_ROW0)
                  |  (1u << LED_COL0)
                  |  (1u << LED_COL1)
                  |  (1u << LED_COL2)
                  |  (1u << LED_COL3)
                  |  (1u << LED_COL4);  // Enable GPIO output
  GPIO_OUTPUT_VAL |= (1u << LED_ROW0)
                  |  (1u << LED_COL0)
                  |  (1u << LED_COL1)
                  |  (1u << LED_COL2)
                  |  (1u << LED_COL3)
                  |  (1u << LED_COL4);  // Initially clear LED
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  switch (Index) {
  case 0:
    GPIO_OUTPUT_VAL &= ~(1u << LED_COL0);
    break;
  case 1:
    GPIO_OUTPUT_VAL &= ~(1u << LED_COL1);
    break;
  case 2:
    GPIO_OUTPUT_VAL &= ~(1u << LED_COL2);
    break;
  case 3:
    GPIO_OUTPUT_VAL &= ~(1u << LED_COL3);
    break;
  case 4:
    GPIO_OUTPUT_VAL &= ~(1u << LED_COL4);
    break;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  switch (Index) {
  case 0:
    GPIO_OUTPUT_VAL |= (1u << LED_COL0);
    break;
  case 1:
    GPIO_OUTPUT_VAL |= (1u << LED_COL1);
    break;
  case 2:
    GPIO_OUTPUT_VAL |= (1u << LED_COL2);
    break;
  case 3:
    GPIO_OUTPUT_VAL |= (1u << LED_COL3);
    break;
  case 4:
    GPIO_OUTPUT_VAL |= (1u << LED_COL4);
    break;
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  switch (Index) {
  case 0:
    GPIO_OUTPUT_VAL ^= (1u << LED_COL0);
    break;
  case 1:
    GPIO_OUTPUT_VAL ^= (1u << LED_COL1);
    break;
  case 2:
    GPIO_OUTPUT_VAL ^= (1u << LED_COL2);
    break;
  case 3:
    GPIO_OUTPUT_VAL ^= (1u << LED_COL3);
    break;
  case 4:
    GPIO_OUTPUT_VAL ^= (1u << LED_COL4);
    break;
  }
}

/*************************** End of file ****************************/
